import pandas as pd
from geotext import GeoText


class EasyCountryParser:

    """
    EasyCountryParser

    This class provides utilities, based on the data from the
    RESTcountries API and the GeoText natural-language parser library,
    for easily extracting and handling country names and codes.

    PROPERTIES:
        .data       - pandas DataFrame containing RESTcountries data
        .tld_to_a2c - python dict, maps TLDs to 2-character ISO codes
        .tld_to_a3c - python dict, maps TLDs to 3-character ISO codes
        .iso2to3    - python dict, maps 2-character ISO codes to 3
        .iso3to2    - python dict, maps 3-character ISO codes to 2
        .a2c_map    - python dict, maps 2-char ISO codes to full names
        .a3c_map    - python dict, maps 3-char ISO codes to full names

    METHODS:
        .retrieve_country - parses plaintext for extractable 2-character
                            ISO codes for countries (which can then be
                            manipulated using the mappers above)

    """

    def __init__(self, altnames=True):

        """
        __init__ for RESTcountries

        Downloads RESTcountries from V2 /all endpoint into a pandas
        DataFrame (RESTcountries.data)

        KEYWORDS:
            altnames = True - use alternative names for certain
                              countries (see get_countrymaps DocString)

        """

        self.data = pd.read_json("https://restcountries.eu/rest/v2/all")
        self.altnames = altnames

        self.get_codemaps()
        self.get_countrymaps()
        self.get_tld_maps()

    @staticmethod
    def retrieve_country(location_description):

        """
        retrieve_country

        This static method consumes a plain-text description of a
        location and uses the geotext library to match it to a country.
        The best match is returned, if there were any (may return None).

        INPUTS:
            location_description

        OUTPUTS:
            country_code - 2-character ISO-Geocode

        """

        if location_description in ["", None]:
            return None

        cdict = GeoText(location_description).country_mentions

        country_codes = list(cdict.keys())

        return country_codes[0] if len(country_codes) > 0 else None

    def get_codemaps(self):

        """
        get_codemaps

        Sets up two python dictionaries, iso2to3 and iso3to2. These map
        2-character ISO codes to 3-character codes and vice-versa.

        """

        self.iso2to3 = self.data.set_index("alpha2Code").alpha3Code.to_dict()
        self.iso3to2 = self.data.set_index("alpha3Code").alpha2Code.to_dict()

    def get_countrymaps(self):

        """
        get_countrymaps

        Generates mapping dictionaries for 2/3-character ISO codes to
        full names for display. If the altnames property of the class
        is set to True, these names will not all be the ones from the
        RESTcountries API (this alternate set are included for reasons of
        compatibility with other sources). The altnames flag is set to
        True by default and is set when the class is initialised.

        """

        self.a2c_map = self.data.set_index("alpha2Code").name.to_dict()
        self.a3c_map = self.data.set_index("alpha3Code").name.to_dict()

        if self.altnames:  # then reassign names to some countries

            alternates = (
                ("BO", "Bolivia"),
                ("VG", "British Virgin Islands"),
                ("VI", "United States Virgin Islands"),
                ("CV", "Cape Verde"),
                ("CD", "Congo"),
                ("FJ", "Fiji Fiji Islands"),
                ("VA", "Holy See (Vatican City State)"),
                ("CI", "Cote DIvoire"),
                ("IR", "Iran"),
                ("KG", "Kyrgyz Republic"),
                ("LA", "Lao"),
                ("LY", "Libyan Arab Jamahiriya"),
                ("MK", "Macedonia"),
                ("FM", "Micronesia"),
                ("MD", "Moldova"),
                ("PS", "Palestinian Territory"),
                ("RE", "Reunion"),
                ("VC", "Saint Vincent and Grenadines"),
                ("KR", "Korea"),
                ("TZ", "Tanzania"),
                ("GB", "United Kingdom"),
                ("UY", "Uruguay, Eastern Republic of"),
                ("VE", "Venezuela"),
                ("VN", "Vietnam"),
            )

            for iso2, aname in alternates:
                iso3 = self.iso2to3[iso2]
                self.a2c_map[iso2] = aname
                self.a3c_map[iso3] = aname

    def get_tld_maps(self):

        """
        get_tld_maps

        Uses RESTcountries.data to set up two dictionaries that map the
        top-level domains (TLDs) of countries to their 2/3-character
        ISO codes.

        These are set at RESTcountries.tld_to_a2c [_a3c]

        """

        self.tld_to_a2c = (
            self.data.set_index("alpha2Code")
            .topLevelDomain.map(lambda L: L[-1])
            .reset_index()
            .set_index("topLevelDomain")
            .alpha2Code.to_dict()
        )

        self.tld_to_a3c = (
            self.data.set_index("alpha3Code")
            .topLevelDomain.map(lambda L: L[-1])
            .reset_index()
            .set_index("topLevelDomain")
            .alpha3Code.to_dict()
        )

        self.tld_to_a2c[".gov"] = "US"
        self.tld_to_a2c[".wales"] = "GB"
        self.tld_to_a2c[".scot"] = "GB"

        self.tld_to_a3c[".gov"] = "USA"
        self.tld_to_a3c[".wales"] = "GBR"
        self.tld_to_a3c[".scot"] = "GBR"



from os import path

MODULE_NAME = "easy-geoparsing"
PACKAGE_NAME = "easy_geoparsing"

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

def format_requirement(requirement_string):
    """
    Format a [packages] line from the Pipfile as needed for the
    install requires list
    """
    # remove the first "=" sign:
    pkg_name = requirement_string.split("=")[0]
    version_info = "=".join(requirement_string.split("=")[1:])

    rs = pkg_name + version_info
    rs = "".join(rs.split('"')) # remove quotation marks
    rs = "".join(rs.split(" ")) # strip out whitespace

    return rs

def get_requirements():
    """
    Read [packages] from the Pipfile and convert to a requirements list
    """

    with open("Pipfile") as pipfile:
        pip_contents = pipfile.read()

    requirements_body = pip_contents.split("[packages]")[-1]
    requirements = [x for x in requirements_body.split("\n") if len(x) > 0]

    return [format_requirement(req) for req in requirements]

def get_version(package_name):
    """
    Looks for __init__.py in supplied package_name and finds
    the value of __version__
    If not found, returns "0.0.1" and warns you to supply a version
    """

    initfile_path = path.join(package_name, "__init__.py")

    pkg_is_there = path.isdir(package_name)
    file_is_there = path.isfile(initfile_path)

    if not pkg_is_there:
        raise FileNotFoundError(
            "Specified package '{package_name}' does not exist".format(
                package_name=package_name
            )
        )

    if not file_is_there:
        raise FileNotFoundError(
            "No __init__.py found in package '{package_name}'".format(
                package_name=package_name
            )
        )

    with open(initfile_path) as pyinit:
        pyinit_cont = pyinit.read().split('\n')

    processed_contents = [
        l.split('"')[1] for l in pyinit_cont if l.startswith("__version__")
    ]

    if len(processed_contents) == 0:
        print("WARNING: no __version__ set in __init__.py")
        return "0.0.1"

    pkg_version = processed_contents[0]

    return pkg_version

def get_download_url(module_name, package_name):
    """
    Returns the download URL with the correct version
    """
    return "https://github.com/apolitical/{m}/archive/v{v}.tar.gz".format(
        m = module_name,
        v = get_version(package_name)
    )

def get_long_description_from_README():
    """
    Returns the contents of README.md as a character string
    """

    with open("README.md") as file_object:
        long_description = file_object.read()
    return long_description

setup(
    name = MODULE_NAME,
    version = get_version(PACKAGE_NAME),
    install_requires = get_requirements(),
    download_url = get_download_url(MODULE_NAME, PACKAGE_NAME),
    packages = [PACKAGE_NAME],
    description = "Easy-to-use module for streamlined parsing of countries from locations",
    long_description = get_long_description_from_README(),
    long_description_content_type="text/markdown",
    author = "PaddyAlton",
    author_email = "paddy.alton@apolitical.co"
)
